/*     Sql2DataCTest.java
 *	
 *     Created on: Mar 23, 2004
 *     Created by: Dennis Groenendijk
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
package test.org.brains2b.sql.toclass;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.sql.Types;
import java.util.Vector;

import junit.framework.TestCase;

import org.brains2b.sql.meta.SqlColumnModel;
import org.brains2b.sql.meta.SqlTableModel;
import org.brains2b.sql.toclass.Sql2DataC;

/** Unit test for  org.brains2b.sql.toclass.Sql2DataC
 * 
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10 [Mar 23, 2004]
 * @see org.brains2b.sql.toclass.Sql2DataC
 */
public class Sql2DataCTest extends TestCase {
	private SqlTableModel tm;
	
	/**
	 * Constructor for Sql2DataCTest.
	 * @param arg0
	 */
	public Sql2DataCTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.swingui.TestRunner.run(Sql2DataCTest.class);
	}

	public void setUp() {
		tm=new SqlTableModel(null,"SCOTT","DEP","TABLE",null);
		SqlColumnModel col1=new SqlColumnModel(null,"SCOTT","DEP","DEPNO",(short) Types.DECIMAL,"NUMBER",2,0,"TRUE",null,null,1);
		SqlColumnModel col2=new SqlColumnModel(null,"SCOTT","DEP","DNAME",(short) Types.VARCHAR,"VARCHAR2",14,0,"TRUE",null,null,2);
		SqlColumnModel col3=new SqlColumnModel(null,"SCOTT","DEP","LOC",(short) Types.VARCHAR,"VARCHAR2",13,0,"TRUE",null,null,3);
		Vector columns=new Vector();
		
		// for generating a DataControl class you need a primary key, pseudo key or ROWID
		col1.setPrimaryKey(1);
		tm.setPrimaryKey(true);
		
		columns.add(col1);
		columns.add(col2);
		columns.add(col3);
		tm.setColumns(columns);
	}
	/*
	 * Test for void Sql2DataC(File, String)
	 */
	public void testSql2DataCFileString() {
		Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac");
		datac.setTableModel(tm);
		datac.run();
		comparefiles(new File("test/sql2datac/DepDC01.java"),datac.getResultfile());
	}

	/*
	 * Test for void Sql2DataC(File, String, SqlTableModel)
	 */
	public void testSql2DataCFileStringSqlTableModel() {
		Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac",tm);
		datac.run();
		comparefiles(new File("test/sql2datac/DepDC01.java"),datac.getResultfile());
	}

	public void testSetReadOnly() {
		Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac");
		datac.setTableModel(tm);
		datac.setReadOnly(true);
		datac.run();
		comparefiles(new File("test/sql2datac/DepDC02.java"),datac.getResultfile());
	}
	
	public void testSetView() {
			SqlTableModel view=new SqlTableModel(null,"SCOTT","DEP_VIEW","VIEW",null);
			view.setColumns(tm.getColumns());
			Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac");
			datac.setTableModel(view);
			datac.run();
			comparefiles(new File("test/sql2datac/DepViewDC.java"),datac.getResultfile());
		}

	public void testSetOptimisticLock() {
		Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac");
		datac.setTableModel(tm);
		datac.setOptimisticLock(true);
		datac.run();
		comparefiles(new File("test/sql2datac/DepDC03.java"),datac.getResultfile());
	}
	
	public void testRowId() {
		((SqlColumnModel) tm.getColumns().get(0)).setPrimaryKey(0);
		SqlColumnModel rowId=new SqlColumnModel(null,"SCOTT","DEP","ROWID",(short) Types.OTHER,"ROWID",0,0,null,null,null,4);
		rowId.setPrimaryKey(1);
		tm.getColumns().add(rowId);
		Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac");
		datac.setTableModel(tm);
		datac.run();
		comparefiles(new File("test/sql2datac/DepDC04.java"),datac.getResultfile());
	}
	
	public void testDoublePK() {
		((SqlColumnModel) tm.getColumns().get(1)).setPrimaryKey(2);
		Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac");
		datac.setTableModel(tm);
		datac.run();
		comparefiles(new File("test/sql2datac/DepDC05.java"),datac.getResultfile());
	}

	public void testNoPK() {
		((SqlColumnModel) tm.getColumns().get(0)).setPrimaryKey(0);
		Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac");
		datac.setTableModel(tm);
		datac.run();
		assertEquals(Sql2DataC.RESULT_ERROR,datac.getResult());
		assertEquals("Table model has no primary key, pseudo primary key or ROWID. It is not possible to generate the DataControl",datac.getMessages().pop());
	}

	private void comparefiles(File left, File right) {
			try {
				BufferedReader lbr=new BufferedReader(new FileReader(left));
				BufferedReader rbr=new BufferedReader(new FileReader(right));
				String lline=lbr.readLine();
				String rline=rbr.readLine();
				int count=0;
				while (lline!=null) {
					count++;
					if (rline!=null) {
						if (lline.indexOf("@since")==-1) { // ignore timestamp in since for string compare
							assertEquals("difference at line:" + count,lline,rline);
						} else {
							assertEquals(lline.indexOf("@since"),rline.indexOf("@since"));
						}
					} else {
						assertNull("difference at line:" + count + ":" + "could not find it in compared file");
					}
					lline=lbr.readLine();
					rline=rbr.readLine();
				}
				assertNull("compared file still contains data",rbr.readLine());
				lbr.close();
				rbr.close();
			} catch (Exception ex) {
				assertTrue(ex.getMessage(),false);
			}
		}
}
